package com.tzld.piaoquan.recommend.server.service.score;



import com.tzld.piaoquan.recommend.server.common.base.UserFeature;
import com.tzld.piaoquan.recommend.server.common.base.VideoRankFeature;
import com.tzld.piaoquan.recommend.server.gen.recommend.RecommendRequest;
import com.tzld.piaoquan.recommend.server.model.Video;
import com.tzld.piaoquan.recommend.server.service.rank.RankParam;
import com.tzld.piaoquan.recommend.server.service.recall.RecallResult;
import com.tzld.piaoquan.recommend.server.service.score.feature.VlogShareFeatureExtractor;
import com.tzld.piaoquan.recommend.server.service.score.model.GBDTModel;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;


public class FeedsShareLTRScorer extends BaseGBDTModelScorer{

    private final static int CORE_POOL_SIZE = 64;
    private final static int TIME_OUT = 150;
    private final static Logger LOGGER = LoggerFactory.getLogger(FeedsShareLTRScorer.class);
    private final static ExecutorService executorService = Executors.newFixedThreadPool(CORE_POOL_SIZE);

    public FeedsShareLTRScorer(ScorerConfigInfo configInfo) {

    }

    private RequestContext getRequestContext(RecommendRequest request) {

    }

    @Override
    public List<Video> scoring(final RecommendRequest recommendRequest,
                               final RankParam param,
                               final RecallResult recallResult,
                               final UserFeature userFeature,
                               final VideoRankFeature videoFeature) {

        feedsShareScore();
        Collections.sort();
        return items;
    }



    public void feedsShareScore(final List<Video> items,
                                       final GBDTModel model,
                                       final UserFeature user,
                                       final RecommendRequest requestData) {
        final int size = items.size();
        if (size == 0) {
            return;
        }
        final Map<String, Double> userFeatures;
        final Map<String, Double> contextFeatures;


        // context feature
        contextFeatures = VlogShareFeatureExtractor.extractContextFeatures(requestContext);
        // user feature
        userFeatures = VlogShareFeatureExtractor.extractUserFeatures();

        // score item
        List<Callable<Object>> callables = new ArrayList<Callable<Object>>();
        for (int index = 0; index < items.size(); index++) {
            final int fIndex = index;
            callables.add(new Callable<Object>() {
                              @Override
                              public Object call() throws Exception {
                                  try {
                                      dwelltimeScore(items.get(fIndex), model, requestContext, userFeatures, contextFeatures, requestData, user);
                                  } catch (Exception e) {
                                      LOGGER.error("dwelltime exception: [{}] [{}]", items.get(fIndex).getId(), ExceptionUtils.getFullStackTrace(e));
                                  }
                                  return new Object();
                              }
                          }
            );
        }

        List<Future<Object>> futures = null;
        try {
            futures = executorService.invokeAll(callables, TIME_OUT, TimeUnit.MILLISECONDS);
        } catch (InterruptedException e) {
            LOGGER.error("execute invoke fail: {}", ExceptionUtils.getFullStackTrace(e));
        }

        //等待所有请求的结果返回, 超时也返回
        if (futures != null) {
            for (Future<Object> future : futures) {
                try {
                    if (future != null && future.isDone() && !future.isCancelled() && future.get() != null) {
                    } else {
                        LOGGER.debug("Canceled Dwelltime Score {}", requestContext.getId());
                    }
                } catch (Exception e) {
                    LOGGER.error("InterruptedException {},{}", ExceptionUtils.getFullStackTrace(e));
                }
            }
        }
    }




    // GBDT计算score
    private void shareScore(final RankerItem item,
                                final GBDTModel model,
                                final RequestContext requestContext,
                                final Map<String, Double> userFeatures,
                                final Map<String, Double> contextFeatures,
                                final int debugLevel,
                                final RecommendRequest requestData, final User user) {
        try {
            //judge null
            if (item == null || item.getItemInfo() == null) {
                return;
            }

            ArticleInfo articleInfo = (ArticleInfo) item.getItemInfo();

            Map<String, Double> articleFeatures;
            double freshness;
            Map<String, Double> features;
            articleFeatures = DurationFeatureExtractor.extractArticleFeatures(articleInfo);


            if (model != null) {
                Map<String, Double> featureScoreMap = new HashMap<String, Double>();
                double pro = model.score(features, featureScoreMap, debugLevel);
                featureScoreMap.put("TOTAL_SCORE", pro);

                pro = pro > 1 ? pro : 1;
                LOGGER.debug("xgb score = {}, lgb score = {}", item.getDwelltimeScore(), pro);

                double lgbWeight = Configuration.getDouble("mivideo-recommend-service.dwelltime_ranker.lgb.weight", 1.0);
                pro = (item.getDwelltimeScore() + lgbWeight * pro) / (1.0 + lgbWeight);
                item.setDwelltimeScore(pro);
                double duration = 0;
                if(item.getItemInfo() != null) {
                    duration = ((ArticleInfo) item.getItemInfo()).getDuration();
                }
                duration = Math.min(Math.max(duration, 1), 1800);
                item.setScore(predictDwelltimeScore(item.getRecScore(), pro, duration, item.getId(), requestData, user));
            }

        } catch (Exception e) {
            LOGGER.error("Exception {},{}", requestContext.getId(), ExceptionUtils.getFullStackTrace(e));
        }
    }












}
